@@ -1,3 +1,13 @@
+/*
+ * Copyright (c) 2001 Sendmail, Inc. and its suppliers.
+ *	All rights reserved.
+ *
+ * By using this file, you agree to the terms and conditions set
+ * forth in the LICENSE file which can be found at the top level of
+ * the sendmail distribution.
+ *
+ */
+
 /*
 **  This program checks to see if your version of setreuid works.
 **  Compile it, make it set-user-ID root, and run it as yourself (NOT as
@@ -13,7 +23,7 @@
 #include <stdio.h>
 
 #ifndef lint
-static char id[] = "@(#)$Id: t_setreuid.c,v 8.7 2001/09/08 01:21:13 gshapiro Exp $";
+static char id[] = "@(#)$Id: t_setreuid.c,v 8.8 2001/09/23 03:35:41 ca Exp $";
 #endif /* ! lint */
 
 #ifdef __hpux
